;*** IRLightBarrier.asm ***;
;------------------------------------------------
; Author : Torsten Knorr
;------------------------------------------------

 .include "tn13def.inc"

;------------------------------------------------

 .macro DEBUG_PULS

    sbi  PORTB,         PORTB3
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    cbi  PORTB,         PORTB3

 .endmacro

;------------------------------------------------

 .def    RI_SREG        = R14
 .def    R_TEMP_L       = R16
 .def    R_TEMP_H       = R17
 .def    R_C_DELAY      = R18
 .def    R_C_RESET      = R19
 .def    RI_TEMP_L      = R20
 .def    RI_TEMP_H      = R21
 .def    R_COUNT_L      = R24
 .def    R_COUNT_H      = R25

;------------------------------------------------

 .cseg
 .org 0x00

    rjmp RESET            ; Reset Handler
    rjmp EXT_INT0         ; IRQ0 Handler
    reti; rjmp PCINT0     ; PCINT0 Handler
    reti; rjmp TIM0_OVF   ; Timer0 Overflow Handler
    reti; rjmp EE_RDY     ; EEPROM Ready Handler
    reti; rjmp ANA_COMP   ; Analog Comparator Handler
    reti; rjmp TIM0_COMPA ; Timer0 CompareA Handler
    reti; rjmp TIM0_COMPB ; Timer0 CompareB Handler
    reti; rjmp WATCHDOG   ; Watchdog Interrupt Handler
    reti; rjmp ADC        ; ADC Conversion Handler

;------------------------------------------------

 EXT_INT0:
    in   RI_SREG,       SREG

; Which status has the engine?
    sbis PINB,          PINB1
    rjmp MOTOR_RUNING

; if pin high
 MOTOR_STOPED:

; The falling edge of INT0 generates an interrupt request.
    in   RI_TEMP_L,     MCUCR
    cbr  RI_TEMP_L,     (1 << ISC00)
    out  MCUCR,         RI_TEMP_L

; If enough feed?
    in   RI_TEMP_L,     TCNT0
    cpi  RI_TEMP_L,     0x3C   ; 60 pieces
    brlo END_OF_INT0

; Send SMS ok.
    sbi  PORTB,         PORTB0
; start count down for reset
    clr  R_C_RESET

    rjmp END_OF_INT0

; if pin Low
 MOTOR_RUNING:

; The rising edge of INT0 generates an interrupt request.
    in   RI_TEMP_L,     MCUCR
    sbr  RI_TEMP_L,     (1 << ISC00)
    out  MCUCR,         RI_TEMP_L

; clear TCNT0
    clr  RI_TEMP_L
    out  TCNT0,         RI_TEMP_L

; reset SMS ok
    cbi  PORTB,         PORTB0

 END_OF_INT0:

    ;DEBUG_PULS
    out  SREG,          RI_SREG
    reti

;------------------------------------------------

; Main program start

 RESET:

; Set Stack Pointer to top of RAM
    ldi  R_TEMP_L,      low(RAMEND)
    out  SPL,           R_TEMP_L

; set PORTB0 to output for IN1 GSM 107 SMS and set to low
    sbi  DDRB,          DDB0
    cbi  PORTB,         PORTB0

; set PORTB1 as input for INT0 and pullup's on (Motor status)
    cbi  DDRB,          DDB1
    sbi  PORTB,         PORTB1

; set PORTB2 as input for T0 and pullup's on (count pieces)
    cbi  DDRB,          DDB2
    sbi  PORTB,         PORTB2

; set PORTB3 as output for debug and set to low
    sbi  DDRB,          DDB3
    cbi  PORTB,         PORTB3

; set PORTB4 as output for LED run and set to low
    sbi  DDRB,          DDB4
    cbi  PORTB,         PORTB4

; set PORTB5 as output and set to high
    sbi  DDRB,          DDB5
    sbi  PORTB,         PORTB5

; set INT0
; The falling edge of INT0 generates an interrupt request.
    in   R_TEMP_L,      MCUCR
    sbr  R_TEMP_L,      (1 << ISC01)
    out  MCUCR,         R_TEMP_L

; clear INT0 flag
    ldi  R_TEMP_L,      (1 << INTF0)
    out  GIFR,          R_TEMP_L

; External Interrupt Request INT0 Enable
    ldi  R_TEMP_L,      (1 << INT0)
    out  GIMSK,         R_TEMP_L

; pinchange interrupts disable
    clr  R_TEMP_L
    out  PCMSK,         R_TEMP_L

; set Timer/Counter normal mode
    out  TCCR0A,        R_TEMP_L

; External clock source on T0 pin. Clock on rising edge.
    ldi  R_TEMP_L,      (1 << CS02) | (1 << CS01) | (1 << CS00)
    out  TCCR0B,        R_TEMP_L

; Enable interrupts
    sei

;------------------------------------------------

 MAIN_LOOP:

;------------------------------------------------
; LED run on

    sbi  PORTB,         PORTB4
    clr  R_COUNT_L
    clr  R_COUNT_H
 LOOP_LED_ON:

    ldi  R_C_DELAY,     0x2D
 LOOP_DELAY_ON:
    dec  R_C_DELAY
    brne LOOP_DELAY_ON

    adiw R_COUNT_L,     0x01
    brne LOOP_LED_ON

;------------------------------------------------
; LED run off

    cbi  PORTB,         PORTB4
    clr  R_COUNT_L
    clr  R_Count_H
 LOOP_LED_OFF:

    ldi  R_C_DELAY,     0x2D
 LOOP_DELAY_OFF:
    dec  R_C_DELAY
    brne LOOP_DELAY_OFF

    adiw R_COUNT_L,     0x01
    brne LOOP_LED_OFF

;------------------------------------------------
; reset SMS ok port after 128 loops

    inc  R_C_RESET
    sbrc R_C_RESET,     7
    cbi  PORTB,         PORTB0

;------------------------------------------------

    rjmp MAIN_LOOP

;------------------------------------------------


